#include "mainwindow.h"
#include "ui_mainwindow.h"

extern "C"
{
    #include "libusblc64.h"
}

void MainWindow::startpipethread()
{
    this->ThPipe = new PipeThread(this);
    this->ThPipe->BufSize = this->packetlength;
    connect(this->ThPipe,SIGNAL(DataChanged(quint16*,qint32,qint32,quint32)),this,SLOT(OnDataChanged(quint16*,qint32,qint32,quint32)));
    connect(this->ThPipe, SIGNAL(finished()),this->ThPipe, SLOT(deleteLater()));
    this->ThPipe->start();
}

void MainWindow::stoppipethread()
{
    if (this->ThPipe->isRunning())
    {
        this->ThPipe->stopthread();
        this->ThPipe->wait();
    }
}

void MainWindow::syncparam()
{
    quint8 u8tmp;
    quint16 u16tmp;
    quint32 u32tmp;
    qint16  i16tmp;
    float ftmp;

    if (ls_getstate(u8tmp,1000) == 0) {
        if (u8tmp == 0) ui->btn_start->setText("start");
        else ui->btn_start->setText("stop");
    }

    if (ls_getmode(u8tmp,1000) == 0) {
        ui->cb_mode->blockSignals(true);
        ui->cb_mode->setCurrentIndex(u8tmp);
        ui->cb_mode->blockSignals(false);
    }

    if (ls_getinttime(u32tmp,1000) == 0) {
        ui->ed_inttime->setText(QString::number(u32tmp));
    }

    if (ls_getadcconfig(u16tmp,1000) == 0) {
        ui->cb_fullscale->blockSignals(true);
        ui->cb_fullscale->setCurrentIndex((u16tmp >> 7) & 0x01);
        ui->cb_fullscale->blockSignals(false);
    }

    if (ls_getadcpga1(u16tmp,1000) == 0) {
        ui->hs_gain->blockSignals(true);
        ui->hs_gain->setValue(u16tmp);
        ui->hs_gain->blockSignals(false);
        ui->lb_gain->setText(QString::number(6 / (1+(5*((63- (float)u16tmp)/63))),'f',4)+" V/V");
    }

    if (ls_getadcoffset1(u16tmp,1000) == 0) {
        i16tmp = u16tmp & 0xFF;
        if ((u16tmp & 0x0100) == 0x0100) i16tmp = i16tmp * (-1);

        ui->hs_offset->blockSignals(true);
        ui->hs_offset->setValue(i16tmp);
        ui->hs_offset->blockSignals(false);

        ftmp = (float)i16tmp * 1.2;
        if (ftmp < -300) ftmp = -300;
        if (ftmp > 300) ftmp = 300;
        ui->lb_offset->setText(QString::number(ftmp,'f',1)+" mV");
    }
}

void MainWindow::init_statusBar()
{
    QStatusBar * bar = ui-> statusBar;

    statusMsg = new QLabel;
    statusMsg-> setMinimumSize (350,20);
    statusMsg-> setFrameShape (QFrame :: WinPanel);
    statusMsg-> setFrameShadow (QFrame :: Sunken);

    statusFPS = new QLabel;
    statusFPS-> setMinimumSize (75,20);
    statusFPS-> setFrameShape (QFrame :: WinPanel);
    statusFPS-> setFrameShadow (QFrame :: Sunken);

    statusMEM = new QLabel;
    statusMEM-> setMinimumSize (80,20);
    statusMEM-> setFrameShape (QFrame :: WinPanel);
    statusMEM-> setFrameShadow (QFrame :: Sunken);

    bar-> addWidget (statusMsg);
    bar-> addWidget (statusFPS);
    bar-> addWidget (statusMEM);

    statusMsg->setText ("USB LINE CAMERA");
    statusFPS->setText ("FPS: 0");
    statusMEM->setText ("MEM: 0");
}

void MainWindow::init_chart(quint16 pixelcount)
{
    plot->setAxisScale( QwtPlot::xBottom, 0, pixelcount-1);
    points->clear();
    for(int i=0;i<pixelcount;i++){
        points->push_back(QPointF(i,0));
    }
    seriesdata->setSamples(*points);
    plot->replot();
}

void MainWindow::create_chart(quint16 pixelcount)
{
    plot = new QwtPlot(ui->plotwidget);
    plot->setTitle( "USB Line Camera" );
    plot->setCanvasBackground( Qt::white );
    plot->setAxisScale( QwtPlot::yLeft, 0, 65535);
    plot->setAxisScale( QwtPlot::xBottom, 0, pixelcount-1);

    grid = new QwtPlotGrid();
    grid->attach(plot);

    curve = new QwtPlotCurve("Curve 1");
    curve->setPen( Qt::blue, 0 ),
    curve->setRenderHint( QwtPlotItem::RenderAntialiased, true );

    seriesdata = new QwtPointSeriesData;
    points = new QVector<QPointF>;

    for(int i=0;i<pixelcount;i++){
        points->push_back(QPointF(i,0));
    }
    seriesdata->setSamples(*points);
    curve->setData(seriesdata);
    curve->attach(plot);

    plot->resize( 500, 320 );
    plot->replot();
}

void MainWindow::OnDataChanged(quint16* buf, qint32 BytesRead, qint32 BytesAvailable, quint32 fps)
{
    statusMEM->setText("MEM: "+QString::number(BytesAvailable));
    statusFPS->setText("FPS: "+QString::number(fps));

    points->clear();
    for(int i=0; i<BytesRead/2;i++){
        points->push_back(QPointF(i,buf[i]));
    }
    seriesdata->setSamples(*points);
    plot->replot();
}

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    init_statusBar();

    this->wpixelcount = 2048;
    this->packetlength = this->wpixelcount * 2; //2 x 2048 pixel = 4KB Packetlength
    ls_initialize(4*1024*1024,this->packetlength); //4MB PipeSize
    create_chart(this->wpixelcount);
}

MainWindow::~MainWindow()
{
    close_device();
    delete ui;
}

void MainWindow::close_device()
{
    if (ls_currentdeviceindex()>= 0){
        this->stoppipethread();
        ls_closedevice();
        ui->btn_open->setText("open");
    }
}

void MainWindow::open_device(qint32 index)
{
    qint32 tmp_packetlength;
    quint16 tmp_sensortype;
    quint16 tmp_pixelcount;
    qint32 ierr;

    if (ls_currentdeviceindex()>= 0){
        close_device();
    } else {
        if (ls_devicecount() > 0){
            ierr = ls_opendevicebyindex(index);
            if (ierr == 0){
                ierr = ls_getpacketlength(tmp_packetlength,1000);
                if (ierr == 0) {
                    if (tmp_packetlength != this->packetlength) {
                        this->packetlength = tmp_packetlength;
                        ls_closedevice();
                        ls_setpacketlength(this->packetlength);
                        ierr = ls_opendevicebyindex(index);
                        if (ierr != 0) {
                            statusMsg->setText("OPEN DEVICE: "+QString(ls_geterrorstring(ierr)));
                            return;
                        }
                    }
                } else statusMsg->setText("GET PACKETLENGTH: "+QString(ls_geterrorstring(ierr)));

                ierr = ls_getsensortype(tmp_sensortype,tmp_pixelcount,1000);
                if (ierr == 0) {
                    this->wsensortype = tmp_sensortype;
                    if (tmp_pixelcount != this->wpixelcount) {
                        this->wpixelcount = tmp_pixelcount;
                        init_chart(this->wpixelcount);
                    };
                } else statusMsg->setText("GET SENSORTYPE: "+QString(ls_geterrorstring(ierr)));
                this->syncparam();
                this->startpipethread();
                ui->btn_open->setText("close");
            } else statusMsg->setText("OPEN DEVICE: "+QString(ls_geterrorstring(ierr)));
        }
    }
}

void MainWindow::on_btn_enum_clicked()
{
    quint16 cnt;

    this->close_device();
    cnt = ls_enumdevices();
    if (cnt > 0) {
        statusMsg->setText("ENUM DEVICES: "+QString::number(cnt)+" DEVICE(S) FOUND");
        for(int i=0;i<cnt;i++){
            ui->cb_devices->addItem(QString(ls_getproductname(i))+" "+QString(ls_getserialnumber(i)));
        }
        ui->cb_devices->setCurrentIndex(0);
        this->open_device(ui->cb_devices->currentIndex());
    } else statusMsg->setText("ENUM DEVICES: NO DEVICES FOUND");
}

void MainWindow::on_btn_open_clicked()
{
    this->open_device(ui->cb_devices->currentIndex());
}

void MainWindow::on_btn_start_clicked()
{
    quint8 state;
    quint8 mode;
    qint32 ierr;

    ierr = ls_getstate(state,1000);
    if (ierr == 0){
        state = (state & 0x01) ^ 0x01;
        ierr = ls_setstate(state,1000);
        if (ierr == 0){
            ls_getmode(mode,1000);
            if ((state == 0) || (mode == 0)) ui->btn_start->setText("start");
            else ui->btn_start->setText("stop");
        }
    }
    statusMsg->setText("SET STATE: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_inttime_clicked()
{
    qint32 ierr;
    quint32 u32tmp;

    ierr = ls_setinttime(ui->ed_inttime->text().toInt(),1000);
    if (ierr == 0) {
        if (ls_getinttime(u32tmp,1000) == 0) {
            ui->ed_inttime->setText(QString::number(u32tmp));
        }
    }
    statusMsg->setText("SET INT.TIME: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_hs_gain_valueChanged(int value)
{
    qint32 ierr;
    float fgain;

    ierr = ls_setadcpga1(value,1000);
    statusMsg->setText("SET GAIN: "+QString(ls_geterrorstring(ierr)));

    fgain = 6 / (1+(5*((63- (float)value)/63)));
    ui->lb_gain->setText(QString::number(fgain,'f',4)+" V/V");
}

void MainWindow::on_hs_offset_valueChanged(int value)
{
    float newvalue;
    quint16 wvalue;
    quint16 sign;
    qint32 ierr;

    newvalue = (float)value * 1.2;
    if (newvalue < -300) newvalue = -300;
    if (newvalue > 300) newvalue = 300;

    if (value < 0) sign = 0x0100;
    else sign = 0;
    wvalue = abs(value) | sign;

    ierr = ls_setadcoffset1(wvalue,1000);
    statusMsg->setText("SET OFFSET: "+QString(ls_geterrorstring(ierr)));

    ui->lb_offset->setText(QString::number(newvalue,'f',1)+" mV");
}

void MainWindow::on_cb_mode_currentIndexChanged(int index)
{
    qint32 ierr;

    ierr = ls_setmode(index,1000);
    if ((ierr == 0) && (index == 0))
        ui->btn_start->setText("start");

    statusMsg->setText("SET MODE: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_cb_fullscale_currentIndexChanged(int index)
{
    qint32 ierr;

    ierr = ls_setadcconfig(index << 7,1000);
    statusMsg->setText("FULL SCALE: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_savesettings_clicked()
{
    qint32 ierr;

    ierr = ls_savesettings(1000);
    statusMsg->setText("SAVE SETTINGS: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_setscl_clicked()
{
    qint32 ierr;

    ierr = ls_seti2cfreq(ui->cb_scl->currentIndex(),1000);
    statusMsg->setText("SET I2C SCL: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_getscl_clicked()
{
    qint32 ierr;
    quint8 scl;

    ierr = ls_geti2cfreq(scl,1000);
    if (ierr == 0) ui->cb_scl->setCurrentIndex(scl);
    statusMsg->setText("GET I2C SCL: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_i2cstat_clicked()
{
    qint32 ierr;
    quint8 stat;

    ierr = ls_geti2cstat(stat,1000);
    if (ierr == 0) ui->lb_i2cstat->setText(ls_geti2cstring(stat));
    else ui->lb_i2cstat->setText("---");
    statusMsg->setText("GET I2C STAT: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_i2cwr_clicked()
{
    qint32 ierr;
    quint8 slvaddr;
    quint16 wlength;
    quint8 stat;
    QString sdata;
    bool status;
    quint8* i2cdata;

    slvaddr = ("0x"+ui->ed_i2caddr->text()).toInt(&status,16);
    sdata = ui->ed_i2cwr->text();
    wlength = sdata.length() / 2;

    i2cdata  = new quint8[wlength];
    for(int i=0;i<wlength;i++){
        i2cdata[i] = ("0x"+sdata.left(2)).toInt(&status,16);
        sdata.remove(1,2);
    }
   ierr = ls_writei2c(slvaddr,i2cdata,wlength,stat,1000);
   statusMsg->setText("I2C WRITE: "+QString(ls_geterrorstring(ierr)));
   if (ierr == 0) ui->lb_i2cstat->setText(ls_geti2cstring(stat));

   delete[] i2cdata;
}

void MainWindow::on_btn_i2crd_clicked()
{
    qint32 ierr;
    quint8 slvaddr;
    quint16 wlength;
    quint8 stat;
    QString sdata;
    bool status;
    quint8* i2cdata;

    slvaddr = ("0x"+ui->ed_i2caddr->text()).toInt(&status,16);
    wlength = ui->sb_i2crd->value();

    i2cdata  = new quint8[wlength];

   ierr = ls_readi2c(slvaddr,i2cdata,wlength,stat,1000);
   if ((ierr == 0) && (wlength > 0)){
       sdata = "";
       for(int i=0; i< wlength;i++){
           sdata = sdata + "0x"+QString::number(i2cdata[i],16)+" ";
       }
       ui->i2ctxt->appendPlainText(sdata);
   }
   statusMsg->setText("I2C READ: "+QString(ls_geterrorstring(ierr)));
   if (ierr == 0) ui->lb_i2cstat->setText(ls_geti2cstring(stat));
   delete[] i2cdata;
}

void MainWindow::on_btn_i2cclear_clicked()
{
    ui->i2ctxt->clear();
}

void MainWindow::on_btn_getinfo_clicked()
{
    qint32 ierr;
    quint16 u16tmp;
    qint32 i32tmp;
    quint32 u32tmp;
    quint16 sensor;
    quint16 pixel;
    int idx;

    idx = ui->cb_devices->currentIndex();

    ui->lb_vendor->setText(ls_getvendorname(idx));
    ui->lb_product->setText(ls_getproductname(idx));
    ui->lb_serial->setText(ls_getserialnumber(idx));

    u16tmp = ls_getmcu1version(idx);
    ui->lb_mcu1version->setText(QString::number((u16tmp >> 8) & 0xFF,16)+"."+QString::number(u16tmp & 0xFF,16));

    ierr = ls_getmcu2version(u16tmp,1000);
    if (ierr == 0) ui->lb_mcu2version->setText(QString::number((u16tmp >> 8) & 0xFF,16)+"."+QString::number(u16tmp & 0xFF,16));

    ierr = ls_getsensortype(sensor,pixel,1000);
    if (ierr == 0) {
        ui->lb_sensor->setText(QString::number(sensor,16)+" <"+ls_getsensorname(sensor)+">");
        ui->lb_pixel->setText(QString::number(pixel));
    }

    ierr = ls_getmcu2sensortype(sensor,1000);
    if (ierr == 0) ui->lb_mcu2sensor->setText(QString::number(sensor,16)+" <"+ls_getsensorname(sensor)+">");

    ierr = ls_getpacketlength(i32tmp,1000);
    if (ierr == 0) ui->lb_mcu2packet->setText(QString::number(i32tmp)+" Bytes");
    else ui->lb_mcu2packet->setText("?0");

    ierr = ls_customfirmware(u32tmp,1000);
    if (ierr == 0) ui->lb_custom->setText(QString::number(u32tmp));
    else ui->lb_custom->setText("?0");

    u16tmp = ls_libversion();
    ui->lb_libversion->setText(QString::number((u16tmp >> 8) & 0xFF,16)+"."+QString::number(u16tmp & 0xFF,16));
}
